/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acsklystat.h"
#include <qtimer.h>
#include <qsettings.h>
#include <QToolTip>

	char tmBuffer[80];
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acsklystat::acsklystat(QWidget *parent) : QWidget(parent)
{
	// load (or set) path for images
	QSettings settings("fermi", "acs");
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
	}
	else {
		imgpath = "../img";
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	// load master status
	master = false;
	if (settings.contains("master")) {
		master = getenv("DISPLAY")==settings.value("master").toString();
		// cout << master.toStdString() << endl;
	}
master = true;
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// connect to control access server
	try {
		plc_server = new Tango::DeviceProxy("f/access_control/safety");
                // plc_server = new Tango::DeviceProxy("ken/plc/fsafety");
                // plc_server = new Tango::DeviceProxy("booster/accessi/plc_accessi");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	refreshCount = 0;
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// init info connections
	connect(ui.k0_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvks()));
	inhibithvks_process = new QProcess(this);
	connect(ui.k1_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk1()));
	inhibithvk1_process = new QProcess(this);
	connect(ui.k2_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk2()));
	inhibithvk2_process = new QProcess(this);
	connect(ui.k3_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk3()));
	inhibithvk3_process = new QProcess(this);
	connect(ui.k4_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk4()));
	inhibithvk4_process = new QProcess(this);
	connect(ui.k5_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk5()));
	inhibithvk5_process = new QProcess(this);
	connect(ui.k6_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk6()));
	inhibithvk6_process = new QProcess(this);
	connect(ui.k7_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk7()));
	inhibithvk7_process = new QProcess(this);
	connect(ui.k8_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk8()));
	inhibithvk8_process = new QProcess(this);
	connect(ui.k9_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk9()));
	inhibithvk9_process = new QProcess(this);
	connect(ui.k10_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk10()));
	inhibithvk10_process = new QProcess(this);
	connect(ui.k11_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk11()));
	inhibithvk11_process = new QProcess(this);
	connect(ui.k12_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk12()));
	inhibithvk12_process = new QProcess(this);
	connect(ui.k13_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk13()));
	inhibithvk13_process = new QProcess(this);
	connect(ui.k14_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk14()));
	inhibithvk14_process = new QProcess(this);
	connect(ui.k15_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk15()));
	inhibithvk15_process = new QProcess(this);
	connect(ui.k16_infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithvk16()));
	inhibithvk16_process = new QProcess(this);
	// connect(ui.infoHv, SIGNAL(clicked()), this, SLOT(open_inhibithv()));
	inhibithv_process = new QProcess(this);
	connect(ui.k0_infoRf, SIGNAL(clicked()), this, SLOT(open_inhibitrfks()));
	inhibitrfks_process = new QProcess(this);
	connect(ui.k1_infoRf, SIGNAL(clicked()), this, SLOT(open_inhibitrfk1()));
	inhibitrfk1_process = new QProcess(this);
	connect(ui.k2_infoRf, SIGNAL(clicked()), this, SLOT(open_inhibitrfk2()));
	inhibitrfk2_process = new QProcess(this);
	connect(ui.infoRf, SIGNAL(clicked()), this, SLOT(open_inhibitrf()));
	inhibitrf_process = new QProcess(this);
	// init images for info buttons
	ui.k0_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k0_infoRf->setIcon(QPixmap(imgpath+"info.png"));
	ui.k1_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k1_infoRf->setIcon(QPixmap(imgpath+"info.png"));
	ui.k2_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k2_infoRf->setIcon(QPixmap(imgpath+"info.png"));
	ui.k3_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k4_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k5_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k6_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k7_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k8_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k9_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k10_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k11_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k12_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k13_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k14_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k15_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.k16_infoHv->setIcon(QPixmap(imgpath+"info.png"));
	// ui.infoHv->setIcon(QPixmap(imgpath+"info.png"));
	ui.infoRf->setIcon(QPixmap(imgpath+"info.png"));
	// set style
	style = " -style=windows";
}

acsklystat::~acsklystat()
{
}    

int acsklystat::mySetColor(QFrame *widget, const char *color, QPalette::ColorRole role=QPalette::Dark)
{
	QPalette button;
	button.setColor(QPalette::Light, QColor(color));
	button.setColor(role, QColor(color));
	widget->setPalette(button);
	return 0;
}

int acsklystat::mySetTextColor(QLabel *widget, const char *text, const char *color)
{
	QPalette button;
	widget->setText(text);
	button.setColor(QPalette::WindowText, QColor(color));
	widget->setPalette(button);
	return 0;
}
    
int acsklystat::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acsklystat::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acsklystat::myWriteCommand(const char *cmdName)
{
	try {
		plc_server->command_inout(cmdName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: " << cmdName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command: " << cmdName << endl;
	}
	return 0;
}

void acsklystat::refresh()
{
	vector<bool> stat, stat2;
	// stat.size(16);
	refreshCount = 1 - refreshCount;
	Tango::DeviceAttribute StateFlags;
	// tunnel watch hinibit
	if (myReadBoolArray("Linac_mod_hv", stat)) {
		mySetTextColor(ui.k0_hv, stat[0]? "HV OFF": "HV ON", stat[0]? "#777777": "#000000");
		mySetTextColor(ui.k1_hv, stat[1]? "HV OFF": "HV ON", stat[1]? "#777777": "#000000");
		mySetTextColor(ui.k2_hv, stat[2]? "HV OFF": "HV ON", stat[2]? "#777777": "#000000");
		mySetTextColor(ui.k3_hv, stat[3]? "HV OFF": "HV ON", stat[3]? "#777777": "#000000");
		mySetTextColor(ui.k4_hv, stat[4]? "HV OFF": "HV ON", stat[4]? "#777777": "#000000");
		mySetTextColor(ui.k5_hv, stat[5]? "HV OFF": "HV ON", stat[5]? "#777777": "#000000");
		mySetTextColor(ui.k6_hv, stat[6]? "HV OFF": "HV ON", stat[6]? "#777777": "#000000");
		mySetTextColor(ui.k7_hv, stat[7]? "HV OFF": "HV ON", stat[7]? "#777777": "#000000");
		mySetTextColor(ui.k8_hv, stat[8]? "HV OFF": "HV ON", stat[8]? "#777777": "#000000");
		mySetTextColor(ui.k9_hv, stat[9]? "HV OFF": "HV ON", stat[9]? "#777777": "#000000");
		mySetTextColor(ui.k10_hv, stat[10]? "HV OFF": "HV ON", stat[10]? "#777777": "#000000");
		mySetTextColor(ui.k11_hv, stat[11]? "HV OFF": "HV ON", stat[11]? "#777777": "#000000");
		mySetTextColor(ui.k12_hv, stat[12]? "HV OFF": "HV ON", stat[12]? "#777777": "#000000");
		mySetTextColor(ui.k13_hv, stat[13]? "HV OFF": "HV ON", stat[13]? "#777777": "#000000");
		mySetTextColor(ui.k14_hv, stat[14]? "HV OFF": "HV ON", stat[14]? "#777777": "#000000");
		mySetTextColor(ui.k15_hv, stat[15]? "HV OFF": "HV ON", stat[15]? "#777777": "#000000");
		mySetTextColor(ui.k16_hv, stat[16]? "HV OFF": "HV ON", stat[16]? "#777777": "#000000");
	}
	if (myReadBoolArray("Linac_mod_rf", stat)) {
		if (myReadBoolArray("Linac_mod_positions", stat2)) {
			mySetTextColor(ui.k0_rf, stat[0]? "RF OFF": (!stat2[0]? "RF ON": "RF on Load"), (stat[0] || stat2[0])? "#00a000": "#c00000");
			mySetTextColor(ui.k1_rf, stat[1]? "RF OFF": (!stat2[1]? "RF ON": "RF on Load"), (stat[1] || stat2[1])? "#00a000": "#c00000");
			mySetTextColor(ui.k2_rf, stat[2]? "RF OFF": (!stat2[2]? "RF ON": "RF on Load"), (stat[2] || stat2[2])? "#00a000": "#c00000");
		}
		else {
			mySetTextColor(ui.k0_rf, "???", "#000000");
			mySetTextColor(ui.k1_rf, "???", "#000000");
			mySetTextColor(ui.k2_rf, "???", "#000000");
		}
		mySetTextColor(ui.k3_rf, stat[3]? "RF OFF": "RF ON", stat[3]? "#00a000": "#c00000");
		mySetTextColor(ui.k4_rf, stat[4]? "RF OFF": "RF ON", stat[4]? "#00a000": "#c00000");
		mySetTextColor(ui.k5_rf, stat[5]? "RF OFF": "RF ON", stat[5]? "#00a000": "#c00000");
		mySetTextColor(ui.k6_rf, stat[6]? "RF OFF": "RF ON", stat[6]? "#00a000": "#c00000");
		mySetTextColor(ui.k7_rf, stat[7]? "RF OFF": "RF ON", stat[7]? "#00a000": "#c00000");
		mySetTextColor(ui.k8_rf, stat[8]? "RF OFF": "RF ON", stat[8]? "#00a000": "#c00000");
		mySetTextColor(ui.k9_rf, stat[9]? "RF OFF": "RF ON", stat[9]? "#00a000": "#c00000");
		mySetTextColor(ui.k10_rf, stat[10]? "RF OFF": "RF ON", stat[10]? "#00a000": "#c00000");
		mySetTextColor(ui.k11_rf, stat[11]? "RF OFF": "RF ON", stat[11]? "#00a000": "#c00000");
		mySetTextColor(ui.k12_rf, stat[12]? "RF OFF": "RF ON", stat[12]? "#00a000": "#c00000");
		mySetTextColor(ui.k13_rf, stat[13]? "RF OFF": "RF ON", stat[13]? "#00a000": "#c00000");
		mySetTextColor(ui.k14_rf, stat[14]? "RF OFF": "RF ON", stat[14]? "#00a000": "#c00000");
		mySetTextColor(ui.k15_rf, stat[15]? "RF OFF": "RF ON", stat[15]? "#00a000": "#c00000");
		mySetTextColor(ui.k16_rf, stat[15]? "RF OFF": "RF ON", stat[16]? "#00a000": "#c00000");
	}
	else {
		mySetTextColor(ui.k0_rf, "???", "#000000");
		mySetTextColor(ui.k1_rf, "???", "#000000");
		mySetTextColor(ui.k2_rf, "???", "#000000");
		mySetTextColor(ui.k3_rf, "???", "#000000");
		mySetTextColor(ui.k4_rf, "???", "#000000");
		mySetTextColor(ui.k5_rf, "???", "#000000");
		mySetTextColor(ui.k6_rf, "???", "#000000");
		mySetTextColor(ui.k7_rf, "???", "#000000");
		mySetTextColor(ui.k8_rf, "???", "#000000");
		mySetTextColor(ui.k9_rf, "???", "#000000");
		mySetTextColor(ui.k10_rf, "???", "#000000");
		mySetTextColor(ui.k11_rf, "???", "#000000");
		mySetTextColor(ui.k12_rf, "???", "#000000");
		mySetTextColor(ui.k13_rf, "???", "#000000");
		mySetTextColor(ui.k14_rf, "???", "#000000");
		mySetTextColor(ui.k15_rf, "???", "#000000");
		mySetTextColor(ui.k16_rf, "???", "#000000");
	}
	if (myReadBoolArray("Linac_mod_abi_hv", stat)) {
		mySetTextColor(ui.k0_hvAbi, stat[0]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k0_hvAbi, stat[0]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k1_hvAbi, stat[1]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k1_hvAbi, stat[1]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k2_hvAbi, stat[2]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k2_hvAbi, stat[2]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k3_hvAbi, stat[3]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k3_hvAbi, stat[3]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k4_hvAbi, stat[4]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k4_hvAbi, stat[4]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k5_hvAbi, stat[5]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k5_hvAbi, stat[5]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k6_hvAbi, stat[6]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k6_hvAbi, stat[6]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k7_hvAbi, stat[7]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k7_hvAbi, stat[7]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k8_hvAbi, stat[8]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k8_hvAbi, stat[8]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k9_hvAbi, stat[9]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k9_hvAbi, stat[9]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k10_hvAbi, stat[10]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k10_hvAbi, stat[10]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k11_hvAbi, stat[11]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k11_hvAbi, stat[11]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k12_hvAbi, stat[12]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k12_hvAbi, stat[12]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k13_hvAbi, stat[13]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k13_hvAbi, stat[13]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k14_hvAbi, stat[14]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k14_hvAbi, stat[14]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k15_hvAbi, stat[15]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k15_hvAbi, stat[15]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k16_hvAbi, stat[16]? "HV Abi": "HV No Abi", "#000000");
		mySetColor(ui.k16_hvAbi, stat[16]? "#ffffff": "#c8ffff", QPalette::Window);
		/*
		mySetTextColor(ui.hvAbi, stat[3]? 
		  "HV Abi (K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15)": 
		  "HV No Abi (K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15)", "#000000");
		mySetColor(ui.hvAbi, stat[3]? "#ffffff": "#c8ffff", QPalette::Window);
		*/
	}
	if (myReadBoolArray("Linac_mod_abi_rf", stat)) {
		mySetTextColor(ui.k0_rfAbi, stat[0]? "RF Abi": "RF No Abi", "#000000");
		mySetColor(ui.k0_rfAbi, stat[0]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k1_rfAbi, stat[1]? "RF Abi": "RF No Abi", "#000000");
		mySetColor(ui.k1_rfAbi, stat[1]? "#ffffff": "#c8ffff", QPalette::Window);
		mySetTextColor(ui.k2_rfAbi, stat[2]? "RF Abi": "RF No Abi", "#000000");
		mySetColor(ui.k2_rfAbi, stat[2]? "#ffffff": "#c8ffff", QPalette::Window);		
		mySetTextColor(ui.rfAbi, stat[3]? 
		  "RF Abi (K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15, K16)": 
		  "RF No Abi (K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15, K16)", "#000000");
		mySetColor(ui.rfAbi, stat[3]? "#ffffff": "#c8ffff", QPalette::Window);
	}
	// set RF pick-up status
	if (myReadBoolArray("Linac_pickup", stat)) {
		mySetTextColor(ui.pickup_ctf1, stat[4]? "RF OFF": "RF ON", stat[4]? "#00a000": "#c00000");
		mySetTextColor(ui.pickup_ctf2, stat[5]? "RF OFF": "RF ON", stat[5]? "#00a000": "#c00000");
		mySetTextColor(ui.pickup_pcgun, stat[0]? "RF OFF": "RF ON", stat[0]? "#00a000": "#c00000");
		mySetTextColor(ui.pickup_s0a, stat[1]? "RF OFF": "RF ON", stat[1]? "#00a000": "#c00000");
		mySetTextColor(ui.pickup_s0b, stat[2]? "RF OFF": "RF ON", stat[2]? "#00a000": "#c00000");
		mySetTextColor(ui.pickup_lerfd, stat[3]? "RF OFF": "RF ON", stat[3]? "#00a000": "#c00000");
	}
	/* test 
	ui.k3_hv->setText(refreshCount? "HV ON": "HV OFF");
	mySetTextColor(ui.k8_rf, refreshCount? "RF ON": "RF OFF", refreshCount? "#c00000": "#00a000");
	mySetTextColor(ui.k0_hvAbi, refreshCount? "HV Abi": "HV No Abi", "#000000");
	mySetColor(ui.k0_hvAbi, refreshCount? "#ffffff": "#c8ffff", QPalette::Window);
	*/
}

void acsklystat::open_inhibithvks()
{
	if (inhibithvks_process->state() == QProcess::Running) {
		inhibithvks_process->kill();
		inhibithvks_process->waitForFinished(3000);			
	}	
	inhibithvks_process->start("acsinhibithvks"+style);

}

void acsklystat::open_inhibithvk1()
{
	if (inhibithvk1_process->state() == QProcess::Running) {
		inhibithvk1_process->kill();
		inhibithvk1_process->waitForFinished(3000);			
	}	
	inhibithvk1_process->start("acsinhibithvk1"+style);

}

void acsklystat::open_inhibithvk2()
{
	if (inhibithvk2_process->state() == QProcess::Running) {
		inhibithvk2_process->kill();
		inhibithvk2_process->waitForFinished(3000);			
	}	
	inhibithvk2_process->start("acsinhibithvk2"+style);

}

void acsklystat::open_inhibithvk3()
{
	if (inhibithvk3_process->state() == QProcess::Running) {
		inhibithvk3_process->kill();
		inhibithvk3_process->waitForFinished(3000);			
	}	
	inhibithvk3_process->start("acsinhibithvk3"+style);

}

void acsklystat::open_inhibithvk4()
{
	if (inhibithvk4_process->state() == QProcess::Running) {
		inhibithvk4_process->kill();
		inhibithvk4_process->waitForFinished(3000);			
	}	
	inhibithvk4_process->start("acsinhibithvk4"+style);

}

void acsklystat::open_inhibithvk5()
{
	if (inhibithvk5_process->state() == QProcess::Running) {
		inhibithvk5_process->kill();
		inhibithvk5_process->waitForFinished(3000);			
	}	
	inhibithvk5_process->start("acsinhibithvk5"+style);

}

void acsklystat::open_inhibithvk6()
{
	if (inhibithvk6_process->state() == QProcess::Running) {
		inhibithvk6_process->kill();
		inhibithvk6_process->waitForFinished(3000);			
	}	
	inhibithvk6_process->start("acsinhibithvk6"+style);

}

void acsklystat::open_inhibithvk7()
{
	if (inhibithvk7_process->state() == QProcess::Running) {
		inhibithvk7_process->kill();
		inhibithvk7_process->waitForFinished(3000);			
	}	
	inhibithvk7_process->start("acsinhibithvk7"+style);

}

void acsklystat::open_inhibithvk8()
{
	if (inhibithvk8_process->state() == QProcess::Running) {
		inhibithvk8_process->kill();
		inhibithvk8_process->waitForFinished(3000);			
	}	
	inhibithvk8_process->start("acsinhibithvk8"+style);

}

void acsklystat::open_inhibithvk9()
{
	if (inhibithvk9_process->state() == QProcess::Running) {
		inhibithvk9_process->kill();
		inhibithvk9_process->waitForFinished(3000);			
	}	
	inhibithvk9_process->start("acsinhibithvk9"+style);

}

void acsklystat::open_inhibithvk10()
{
	if (inhibithvk10_process->state() == QProcess::Running) {
		inhibithvk10_process->kill();
		inhibithvk10_process->waitForFinished(3000);			
	}	
	inhibithvk10_process->start("acsinhibithvk10"+style);

}

void acsklystat::open_inhibithvk11()
{
	if (inhibithvk11_process->state() == QProcess::Running) {
		inhibithvk11_process->kill();
		inhibithvk11_process->waitForFinished(3000);			
	}	
	inhibithvk11_process->start("acsinhibithvk11"+style);

}

void acsklystat::open_inhibithvk12()
{
	if (inhibithvk12_process->state() == QProcess::Running) {
		inhibithvk12_process->kill();
		inhibithvk12_process->waitForFinished(3000);			
	}	
	inhibithvk12_process->start("acsinhibithvk12"+style);

}

void acsklystat::open_inhibithvk13()
{
	if (inhibithvk13_process->state() == QProcess::Running) {
		inhibithvk13_process->kill();
		inhibithvk13_process->waitForFinished(3000);			
	}	
	inhibithvk13_process->start("acsinhibithvk13"+style);

}

void acsklystat::open_inhibithvk14()
{
	if (inhibithvk14_process->state() == QProcess::Running) {
		inhibithvk14_process->kill();
		inhibithvk14_process->waitForFinished(3000);			
	}	
	inhibithvk14_process->start("acsinhibithvk14"+style);

}

void acsklystat::open_inhibithvk15()
{
	if (inhibithvk15_process->state() == QProcess::Running) {
		inhibithvk15_process->kill();
		inhibithvk15_process->waitForFinished(3000);			
	}	
	inhibithvk15_process->start("acsinhibithvk15"+style);

}

void acsklystat::open_inhibithvk16()
{
	if (inhibithvk16_process->state() == QProcess::Running) {
		inhibithvk16_process->kill();
		inhibithvk16_process->waitForFinished(3000);			
	}	
	inhibithvk16_process->start("acsinhibithvk16"+style);

}

void acsklystat::open_inhibithv()
{
	if (inhibithv_process->state() == QProcess::Running) {
		inhibithv_process->kill();
		inhibithv_process->waitForFinished(3000);			
	}	
	inhibithvks_process->start("acsinhibithv"+style);

}

void acsklystat::open_inhibitrfks()
{
	if (inhibitrfks_process->state() == QProcess::Running) {
		inhibitrfks_process->kill();
		inhibitrfks_process->waitForFinished(3000);			
	}	
	inhibitrfks_process->start("acsinhibitrfks"+style);

}

void acsklystat::open_inhibitrfk1()
{
	if (inhibitrfk1_process->state() == QProcess::Running) {
		inhibitrfk1_process->kill();
		inhibitrfk1_process->waitForFinished(3000);			
	}	
	inhibitrfk1_process->start("acsinhibitrfk1"+style);

}

void acsklystat::open_inhibitrfk2()
{
	if (inhibitrfk2_process->state() == QProcess::Running) {
		inhibitrfk2_process->kill();
		inhibitrfk2_process->waitForFinished(3000);			
	}	
	inhibitrfk2_process->start("acsinhibitrfk2"+style);

}

void acsklystat::open_inhibitrf()
{
	if (inhibitrf_process->state() == QProcess::Running) {
		inhibitrf_process->kill();
		inhibitrf_process->waitForFinished(3000);			
	}	
	inhibitrf_process->start("acsinhibitrf"+style);

}
